/* Copyright 2012 Sam Bull */

const Gio = imports.gi.Gio;
const Gtk = imports.gi.Gtk;
const St = imports.gi.St;

const Main = imports.ui.main;
const IconGrid = imports.ui.iconGrid;
const Lang = imports.lang;
const Dash = imports.ui.dash;

const Extension = imports.misc.extensionUtils.getCurrentExtension();
const NotesView = Extension.imports.view.NotesView;
const NotesIconPath = Extension.path + '/notes-icon.svg';

// For _adjustIconSize
const Clutter = imports.gi.Clutter;
const Tweener = imports.ui.tweener;
const DASH_ANIMATION_TIME = Dash.DASH_ANIMATION_TIME;

let dash, viewSelector;
let _a11yFocusPageOld, _adjustIconSizeOld;

let showNotesIcon;
let notesPage;
let signal_notes, signal_apps, signal_show, signal_hide;

/* The button to be added to the dash. */
const ShowNotesIcon = new Lang.Class({
    Name: 'ShowNotesIcon',
    Extends: Dash.DashItemContainer,

    _init: function() {
        this.parent();

        this.toggleButton = new St.Button({ style_class: 'show-apps',
                                            track_hover: true,
                                            can_focus: true,
                                            toggle_mode: true });
        this._iconActor = null;
        this.icon = new IconGrid.BaseIcon(_("Show Notes"),
                                           { setSizeManually: true,
                                             showLabel: false,
                                             createIcon: Lang.bind(
                                                this, this._createIcon) });
        this.toggleButton.add_actor(this.icon.actor);
        this.toggleButton._delegate = this;

        this.setChild(this.toggleButton);
    },

    _createIcon: function(size) {
        this._iconActor = new St.Icon({ gicon: Gio.icon_new_for_string(
                                            NotesIconPath),
                                        icon_size: size,
                                        style_class: 'show-apps-icon',
                                        track_hover: true });
        return this._iconActor;
    }
});

/* Replace Dash's method to update notes icon size. */
function _adjustIconSize() {
    // For the icon size, we only consider children which are "proper"
    // icons (i.e. ignoring drag placeholders) and which are not
    // animating out (which means they will be destroyed at the end of
    // the animation)
    let iconChildren = this._box.get_children().filter(function(actor) {
        return actor._delegate.child &&
               actor._delegate.child._delegate &&
               actor._delegate.child._delegate.icon &&
               !actor._delegate.animatingOut;
    });

    //iconChildren.push(this._showAppsIcon.actor); // Only removing this line..
    iconChildren.push(showNotesIcon.actor);  // ..and adding this one.

    if (this._maxHeight == -1)
        return;

    let themeNode = this._container.get_theme_node();
    let maxAllocation = new Clutter.ActorBox({ x1: 0, y1: 0,
                                               x2: 42 /* whatever */,
                                               y2: this._maxHeight });
    let maxContent = themeNode.get_content_box(maxAllocation);
    let availHeight = maxContent.y2 - maxContent.y1;
    let spacing = themeNode.get_length('spacing');


    let firstIcon = iconChildren[0]._delegate.child._delegate.icon;

    let minHeight, natHeight;

    // Enforce the current icon size during the size request if
    // the icon is animating
    if (firstIcon._animating) {
        let [currentWidth, currentHeight] = firstIcon.icon.get_size();

        firstIcon.icon.set_size(this.iconSize, this.iconSize);
        [minHeight, natHeight] = iconChildren[0].get_preferred_height(-1);

        firstIcon.icon.set_size(currentWidth, currentHeight);
    } else {
        [minHeight, natHeight] = iconChildren[0].get_preferred_height(-1);
    }

    // Subtract icon padding and box spacing from the available height
    availHeight -= iconChildren.length * (natHeight - this.iconSize) +
                   (iconChildren.length - 1) * spacing;

    let availSize = availHeight / iconChildren.length;

    let iconSizes = [ 16, 22, 24, 32, 48, 64 ];

    let newIconSize = 16;
    for (let i = 0; i < iconSizes.length; i++) {
        if (iconSizes[i] < availSize)
            newIconSize = iconSizes[i];
    }

    if (newIconSize == this.iconSize)
        return;

    let oldIconSize = this.iconSize;
    this.iconSize = newIconSize;
    this.emit('icon-size-changed');

    let scale = oldIconSize / newIconSize;
    for (let i = 0; i < iconChildren.length; i++) {
        let icon = iconChildren[i]._delegate.child._delegate.icon;

        // Set the new size immediately, to keep the icons' sizes
        // in sync with this.iconSize
        icon.setIconSize(this.iconSize);

        // Don't animate the icon size change when the overview
        // is transitioning, not visible or when initially filling
        // the dash
        if (!Main.overview.visible || Main.overview.animationInProgress ||
            !this._shownInitially)
            continue;

        let [targetWidth, targetHeight] = icon.icon.get_size();

        // Scale the icon's texture to the previous size and
        // tween to the new size
        icon.icon.set_size(icon.icon.width * scale,
                           icon.icon.height * scale);

        icon._animating = true;
        Tweener.addTween(icon.icon,
                         { width: targetWidth,
                           height: targetHeight,
                           time: DASH_ANIMATION_TIME,
                           transition: 'easeOutQuad',
                           onComplete: function() {
                               icon._animating = false;
                           }
                         });
    }
}

/* Replace ViewSelector's method to allow notes view to be displayed. */
function onButtonToggled() {
    if (viewSelector.active)
        viewSelector.reset();
    else if (dash.showAppsButton.checked)
        viewSelector._showPage(viewSelector._appsPage);
    else if (showNotesIcon.toggleButton.checked)
        viewSelector._showPage(notesPage);
    else
        viewSelector._showPage(viewSelector._workspacesPage);
}

/* Replace ViewSelector's to allow notes view to be selected with CtrlAltTab. */
function _a11yFocusPage(page) {
    this._showAppsButton.checked = page == this._appsPage;
    showNotesIcon.toggleButton.checked = page == notesPage;
    page.navigate_focus(null, Gtk.DirectionType.TAB_FORWARD, false);
}

function init() {
    dash = Main.overview._dash;
    viewSelector = Main.overview._viewSelector;

    _adjustIconSizeOld = Main.overview._dash._adjustIconSize;
    _a11yFocusPageOld = Main.overview._viewSelector._a11yFocusPage;
}

function enable() {
    // Replace show apps button
    showNotesIcon = new ShowNotesIcon();
    showNotesIcon.icon.setIconSize(dash.iconSize);
    dash._hookUpLabel(showNotesIcon);

    dash._container.remove_actor(dash._showAppsIcon.actor);
    dash._container.add_actor(showNotesIcon.actor);

    // Add notes page
    notesPage = viewSelector._addPage(new NotesView().actor, null, _("Notes"),
                                      'system-run-symbolic');
    notesPage.hide();

    // Rewire viewSelector's behaviour
    signal_notes = showNotesIcon.toggleButton.connect('notify::checked',
                                                      onButtonToggled);
    // TODO Need to disconnect old signal, but there is no mechanism to do so.
    signal_apps = viewSelector._showAppsButton.connect('notify::checked',
                                                       onButtonToggled);
    signal_show = Main.overview.connect('showing',
        function () { showNotesIcon.toggleButton.checked = false; });
    signal_hide = Main.overview.connect('hiding',
        function () { showNotesIcon.toggleButton.checked = false; });
    Main.overview._dash._adjustIconSize = _adjustIconSize;
    viewSelector._a11yFocusPage = _a11yFocusPage;
}

function disable() {
    viewSelector._a11yFocusPage = _a11yFocusPageOld;
    Main.overview._dash._adjustIconSize = _adjustIconSizeOld;
    Main.overview.disconnect(signal_hide);
    Main.overview.disconnect(signal_show);
    viewSelector._showAppsButton.disconnect(signal_apps);
    showNotesIcon.toggleButton.disconnect(signal_notes);

    notesPage.destroy();
    notesPage = null;

    dash._container.remove_actor(showNotesIcon.actor);
    dash._container.add_actor(dash._showAppsIcon.actor);
    showNotesIcon.destroy();
    showNotesIcon = null;
}
